# -*- coding: utf-8 -*-

# module usunits

# Copyright (C) 2019 National Research Council Canada
# Author:  Harold Parks

# This file is part of MetroloPy.

# MetroloPy is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software 
# Foundation, either version 3 of the License, or (at your option) any later 
# version.

# MetroloPy is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more 
# details.

# You should have received a copy of the GNU General Public License along with 
# MetroloPy. If not, see <http://www.gnu.org/licenses/>.

"""
This module is loaded by the gummy.units module and is not intended be be
imported directly.  The gummy.siunits module must be loaded before loading 
this module.

Most units here are from:
    
NIST Special Publication 1038, "The International System of Units (SI) – 
Conversion Factors for General Use", May 2006.
"""

from .unit import _BuiltinLib,Unit,Conversion
from .offsetunit import OffsetUnit,OffsetConversion
from .prefixedunit import PrefixedUnit
from .ummy import MFraction

with _BuiltinLib():
    _in = PrefixedUnit('inch','in',Conversion('m',MFraction('0.0254')),prefixes=['micro'],
                       add_symbol=True,description='unit of length')
    Unit('hand','hand',Conversion(_in,4),add_symbol=True,description='unit of length')
    _ft = Unit('foot','ft',Conversion(_in,12),add_symbol=True,description='unit of length')
    _yd = Unit('yard','yd',Conversion(_ft,3),add_symbol=True,description='unit of length')
    Unit('mile','mi',Conversion(_yd,1760),add_symbol=True,description='unit of length')
    _pica = Unit('pica','P/',Conversion(_in,MFraction(1,6)),add_symbol=True,description='unit of length')
    Unit('point','p',Conversion(_pica,MFraction(1,12)),add_symbol=True,description='unit of length')
    Unit('link','li',Conversion(_ft,MFraction(33,50)),add_symbol=True,description='unit of length')
    Unit('survey foot','ft',Conversion('m',MFraction(1200,3937)),add_symbol=False,description='unit of length')
    _rod = Unit('rod','rd',Conversion(_ft,25),add_symbol=True,description='unit of length')
    _ch = Unit('chain','ch',Conversion(_rod,4),add_symbol=True,description='unit of length')
    _fur = Unit('furlong','fur',Conversion(_ch,10),add_symbol=True,description='unit of length')
    _mi = Unit('survey mile','mi',Conversion(_fur,8),add_symbol=False,description='unit of length')
    Unit.alias('statute mile',_mi)
    Unit('league','lea',Conversion(_mi,3),add_symbol=True,description='unit of length')
    _ftm = Unit('fathom','ftm',Conversion(_yd,2),add_symbol=True,description='unit of length')
    Unit('cable','cb',Conversion(_ftm,120),add_symbol=True,description='unit of length')
    _mil = Unit('thousandth of an inch','mil',Conversion(_in,MFraction(1,1000)),add_symbol=True,description='unit of length')
    Unit.alias('thou',_mil)
    Unit.alias('thousandth',_mil)
          
    _acre = Unit('acre','acre',Conversion(_ch**2,10),add_symbol=True,description='unit of area')
    _section = Unit('section','section',Conversion(_acre,640),add_symbol=True,description='unit of area')
    Unit('survey township','twp',Conversion(_section,36),add_symbol=True,description='unit of area')
    
    Unit('cubic inch','cu in',Conversion(_in**3,1),add_symbol=True,
         short_name='cu-in',description='unit of volume')
    Unit('cubic foot','cu ft',Conversion(_ft**3,1),add_symbol=True,
         short_name='cu-ft',description='unit of volume')
    Unit('cubic yard','cu yd',Conversion(_yd**3,1),add_symbol=True,
         short_name='cu-yd',description='unit of volume')
    Unit('acre-foot','acre ft',Conversion(_ft**3,43560),add_symbol=True,
         short_name='acre-ft',description='unit of volume')
    
    _gal = Unit('gallon','gal',Conversion(_in**3,231),add_symbol=True,description='unit of volume')
    Unit.alias('liquid gallon',_gal)
    Unit.alias('liquid gal',_gal)
    
    _qt = Unit('quart','qt',Conversion(_gal,MFraction(1,4)),add_symbol=True,description='unit of volume')
    Unit.alias('liquid quart',_qt)
    Unit.alias('liquid qt',_qt)
    
    _pt = Unit('pint','pt',Conversion(_qt,MFraction(1,2)),add_symbol=True,description='unit of volume')
    Unit.alias('liquid pint',_pt)
    Unit.alias('liquid pt',_pt)
    
    _cp = Unit('cup','cp',Conversion(_pt,MFraction(1,2)),add_symbol=True,description='unit of volume')
    _gi = Unit('gill','gi',Conversion(_cp,MFraction(1,2)),add_symbol=True,description='unit of volume')
    _floz = Unit('fluid ounce','fl oz',Conversion(_gi,MFraction(1,4)),add_symbol=True,
                 short_name='fl-oz',description='unit of volume')
    _tbsp = Unit('tablespoon','Tbsp',Conversion(_floz,MFraction(1,2)),add_symbol=True,description='unit of volume')
    _tsp = Unit('teaspoon','tsp',Conversion(_tbsp,MFraction(1,3)),add_symbol=True,description='unit of volume')
    _minum = Unit('minim','min',Conversion(_tsp,MFraction('0.125')),add_symbol=False,description='unit of volume')
    Unit('fluid dram','fl dr',Conversion(_minum,60),add_symbol=True,
         short_name='fl-dr',description='unit of volume')
    Unit('shot','jig',Conversion(_tbsp,3),add_symbol=True,description='unit of volume')
    
    _bbl = Unit('barrel','bbl',Conversion(_gal,MFraction('31.5')),add_symbol=True,description='unit of volume')
    Unit.alias('liquid barrel',_bbl)
    Unit.alias('liquid bbl',_bbl)
    
    Unit('oil barrel','bbl',Conversion(_gal,42),add_symbol=False,description='unit of volume')
    Unit('hogshead','hogshead',Conversion(_gal,65),add_symbol=True,description='unit of volume')
    
    _dgal = Unit('dry gallon','gal',Conversion(_in**3,MFraction('268.8025')),add_symbol=False,
         short_name='dry-gal',description='unit of volume')
    _dqt = Unit('dry quart','qt',Conversion(_dgal,MFraction(1,4)),add_symbol=False,
         short_name='dry-qt',description='unit of volume')
    Unit('dry pint','pt',Conversion(_dqt,MFraction(1,2)),add_symbol=False,
         short_name='dry-pt',description='unit of volume')
    _pk = Unit('peck','pk',Conversion(_dgal,2),add_symbol=True,description='unit of volume')
    Unit('bushel','bu',Conversion(_pk,4),add_symbol=True,description='unit of volume')
    Unit('dry barrel','bu',Conversion(_in**3,7056),add_symbol=False,
         short_name='dry-bbl',description='unit of volume')
         
    _lb = PrefixedUnit('pound','lb',Conversion('kg',MFraction('0.45359237')),add_symbol=True,
                       prefixes=['micro'],description='unit of mass')
    Unit.alias('avoirdupois pound',_lb)
    Unit.alias('avdp lb',_lb)
    Unit.alias('lbm',_lb)
    Unit.alias('pound mass',_lb)
    Unit.alias('ulbm','ulb')
    Unit.alias('micropound mass','ulb')
    
    _oz = Unit('ounce','oz',Conversion(_lb,MFraction('0.0625')),add_symbol=True,description='unit of mass')
    Unit.alias('avoirdupois ounce',_oz)
    Unit.alias('avdp oz',_oz)
    
    Unit('dram','dr',Conversion(_oz,MFraction('0.0625')),add_symbol=True,description='unit of mass')
    
    _gr = Unit('grain','gr',Conversion(_lb,MFraction(1,7000)),add_symbol=True,description='unit of mass')
    
    Unit('hundredweight','cwt',Conversion(_lb,100),add_symbol=True,description='unit of mass')
    Unit('long hundredweight','long cwt',Conversion(_lb,112),add_symbol=True,description='unit of mass')
    Unit('short ton','tn',Conversion(_lb,2000),add_symbol=True,description='unit of mass')
    Unit('long ton','long ton',Conversion(_lb,2240),add_symbol=False,description='unit of mass')
    
    _dwt = Unit('pennyweight','dwt',Conversion(_gr,24),add_symbol=True,description='unit of mass')
    _toz = Unit('troy ounce','oz t',Conversion(_dwt,20),add_symbol=True,
         short_name='oz-t',description='unit of mass')
    Unit('troy pound','lb t',Conversion(_toz,12),add_symbol=True,
         short_name='lb-t',description='unit of mass')
         
    _lbf = Unit('pound force','lbf',Conversion('lb m s**-2',MFraction('9.80665')),add_symbol=True,description='unit of force')
    Unit('slug','slug',Conversion('lbf s**2 ft**-1',1),add_symbol=True,description='unit of force')
    
    _degR = Unit('degree Rankine','\u00B0R',Conversion('K',MFraction(5,9)),
                 latex_symbol='^{\circ}R',ascii_symbol='degR',add_symbol=True,
                 description='unit of temperature')
    Unit.alias('degree R',_degR)
    Unit.alias('deg R',_degR)
    
    _degF = OffsetUnit('degree Fahrenheit','\u00B0F',OffsetConversion('degR',MFraction('459.67')),
                latex_symbol='^{\circ}F',ascii_symbol='degF',add_symbol=True,
                description='unit of temperature')
    Unit.alias('degree F',_degF)
    Unit.alias('deg F',_degF)
               
    Unit('pound per square inch','psi',Conversion(_lbf*_in**-2),add_symbol=True,
         description='unit of pressure')
               
    Unit('board-foot','board-foot',Conversion(_ft**2*_in,1),add_symbol=True,
         short_name='board-ft',description='unit of volume')
    
    _cal = Unit('calorie','cal',Conversion('J',MFraction('4.184')),add_symbol=True,
                description='unit of energy')
    Unit.alias('thermochemical calorie',_cal)
    Unit.alias('calth',_cal)
    
    _Cal = Unit('large calorie','Cal',Conversion(_cal,1000),add_symbol=True,
                description='unit of energy')
    Unit.alias('kilocalorie',_Cal)
    Unit.alias('kcal',_Cal)
    Unit.alias('dietary calorie',_Cal)
    Unit.alias('Calorie',_Cal)
    
    _btu = Unit('IT British thermal unit','BTU',Conversion('J',MFraction('1055.05585262')),
                add_symbol=True,description='unit of energy')
    Unit.alias('Btu',_btu)
   
    Unit('horsepower','hp',Conversion('lbf ft s**-1',550), add_symbol=True,
         description='unit of power')
    
    _ftlb = Unit('foot-pound','ft\u00B7lb',Conversion(_ft*_lbf,1),add_symbol=True,
        ascii_symbol='ft-lb',description='unit of work or energy')
    Unit.alias('ft\u00B7lbf',_ftlb)
    Unit.alias('ft-lbf',_ftlb)
    
    _lbft = Unit('pound-foot','lb\u00B7ft',Conversion(_ft*_lbf,1),add_symbol=True,
        ascii_symbol='lb-ft',description='unit of torque')
    Unit.alias('lbf\u00B7ft',_lbft)
    Unit.alias('lbf-lft',_lbft)
    
    Unit('rack unit','U',Conversion(_in,MFraction('1.75')),add_symbol=False,
         description='unit of length')
    Unit('square','square',Conversion(_ft**2,100),add_symbol=False,
         description='unit of area used in construction')
    Unit('gasoline gallon equivalent','gasoline-gallon-equivalent',
         Conversion('kW h',33.7),add_symbol=True,description='unit of energy')
    _ttnt = PrefixedUnit('tons of TNT equivalent','t(TNT)',Conversion('GJ',4.184),add_symbol=True,
         description='unit of energy in an explosion',prefixes=['kilo','mega','giga'])
    Unit.alias('tons of TNT',_ttnt)
    Unit.alias('tons of tnt',_ttnt)



# -*- coding: utf-8 -*-

# module unitutils

# Copyright (C) 2019 National Research Council Canada
# Author:  Harold Parks

# This file is part of MetroloPy.

# MetroloPy is free software: you can redistribute it and/or modify it under
# the terms of the GNU General Public License as published by the Free Software 
# Foundation, either version 3 of the License, or (at your option) any later 
# version.

# MetroloPy is distributed in the hope that it will be useful, but WITHOUT ANY
# WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
# FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more 
# details.

# You should have received a copy of the GNU General Public License along with 
# MetroloPy. If not, see <http://www.gnu.org/licenses/>.

"""
the search_units, shadow_units, and convert functions are defined here
"""
from .gummy import gummy
from .unit import Unit,one
from .printing import PrettyPrinter,print_markdown,print_html,ipython_installed
import numpy as np


def _mrtxt(txt,fmt):
    if fmt == 'latex':
        txt = txt.replace('*','\\*')
        txt = txt.replace('_','\\_')
        txt = txt.replace('#','\\#')
    return txt

class _search_display:
    def __init__(self,search,show_all,units):
        self.search = search
        self.show_all = show_all
        self.units = units
        
    def __repr__(self):
        if PrettyPrinter.printer == 'ascii':
            return search_units(self.search,fmt='ascii',show_all=self.show_all,
                                units=self.units,prnt=False)
        return search_units(self.search,fmt='unicode',show_all=self.show_all,
                                units=self.units,prnt=False)
        
    def _repr_html_(self):
        if PrettyPrinter.printer == 'any' or PrettyPrinter.printer == 'html':
            return search_units(self.search,fmt='html',show_all=self.show_all,
                                units=self.units,prnt=False)
        return None
        
    def _repr_markdown_(self):
        if PrettyPrinter.printer == 'any' or PrettyPrinter.printer == 'latex':
            return search_units(self.search,fmt='latex',show_all=self.show_all,
                                units=self.units,prnt=False)
        return None
    
def search_units(search=None,fmt=None,show_all=False,units=None,prnt=True):
    """
    Prints a list of all loaded units or all units that match the search terms.
    
    Parameters
    ----------
    search: `str` or `None`, optional
        A space separated list of search terms to case insentively match.
        If this is omitted or set equal to `None` then a list of all loaded
        units will be printed.  The default is `None`.

    fmt: {'html','latex','unicode','ascii',`None`},optional
        The output format.  If `None`, then the `gummy.printer` value is used.
        If latex output is selected, Markdown is actually used with the unit
        symbols and conversion displayed using inline LaTeX.

    show_all: `bool`, optional
        If `True` units are shown with each prefix listed on a separate line
        (e.g. the millisecond and the microsecond are listed in addition to
        the second) and interval units are shown. If `False` only the base
        unit is shown.  The default is `False`.

    units: `list` of `str` or `Unit`,optional
        A list of units to print.  If this parameter is specified the values
        of the search and `show_all` parameters are ignored.

    prnt: `bool`, optional
        If this is `True`, the results are printed.  If it is `False` the results
        are returned as a string.  The default is `True`.
    """
    from importlib import import_module
    from .prefixedunit import PrefixedUnit
    from .nonlinearunit import NonlinearUnit
    from .offsetunit import OffsetUnit
    from .logunit import LogUnit
    from .unit import _CompositeUnit
    
    if fmt is None and prnt:
        return _search_display(search,show_all,units)

    fmt = fmt.lower().strip()
    if fmt == 'utf-8':
        fmt = 'unicode'
    
    if units is None:
        while len(Unit._builtins_to_import) > 0:
            import_module(Unit._builtins_to_import.pop(),Unit.__module__)
            
        units = set(Unit._builtin_lib.values()).union(set(Unit._lib.values()))
        
        if search is None:
            if len(units) == 0:
                if prnt:
                    print('no units are loaded')
                    return
                return ''
        else:
            uf = set()
            for u in units:
                s = set()
                for a in u.aliases:
                    s = s.union(set(a.lower().split()))
                for a in u.shadowed_aliases:
                    s = s.union(set(a.lower().split()))
                if not isinstance(u,_CompositeUnit):
                    s = s.union(u.name.lower().split())
                    s = s.union(set(u.symbol.lower().split()))
                    s = s.union(set(u.symbol.lower().split()))
                    if u.ascii_symbol is not None:
                        s = s.union(set(u.ascii_symbol.lower().split()))
                    if u.description is not None:
                        s = s.union(set(u.description.lower().split()))
                        
                s = {i.strip(',.;:()') for i in s}
                
                srch = search.lower().split()
                ad = True
                for a in srch:
                    if a.strip(',.;') not in s:
                        ad = False
                        break
                if ad:
                    uf.add(u)
                    
            uf = uf.union({u for u in units if u.parent in uf})
                    
            units = uf
            if len(units) == 0:
                if prnt:
                    print('no units found matching "' + search + '"')
                    return
                return ''
            
        if not show_all:
            units = [u for u in units if u.parent is None or u.parent not in units]
            
        uf = []
        for u in units:
            if isinstance(u,_CompositeUnit):
                uf += [(a,u) for a in u._aliases]
            else:
                uf.append((u.name,u))
        units = uf
    else:
        if isinstance(units,str) or isinstance(units,Unit):
            units = [units]
        units = [Unit.unit(u) for u in units]
        
        if show_all:
            uf = set(units)
            allunits = set(Unit._builtin_lib.values()).union(set(Unit._lib.values()))
            for u in units:
                if u.parent is None:
                    p = u
                else:
                    p = u.parent
                for a in allunits:
                    if a is p or a.parent is p:
                        uf.add(a)
            units = uf
                    
        units = [(u.name,u) for u in units]

    units = sorted(units,key=lambda u:u[0].lower())
        
    if fmt == 'latex':
        txt = "<ul style=\"font-family: 'Times New Roman', Times, serif;font-size:1.2em\">\n"
    elif fmt == 'html':
        txt = "<ul>\n"
    else:
        txt = ''
    for u in units:
        if fmt in ['latex','html']:
            txt += "<li>"
        try:
            if u[0] != u[1].name:
                txt += u[0] + ', alias for: '
                if isinstance(u[1],_CompositeUnit):
                    stxt = u[1].tostring(fmt=fmt)
                    if fmt == 'latex':
                        txt += '$ ' + stxt + ' $'
                    else:
                        txt += stxt
                else:
                    txt += _mrtxt(u[1].name,fmt)
            else:
                u = u[1]
                txt += u.name
                ttxt = ''
                if isinstance(u,PrefixedUnit) and not show_all:
                    if not show_all:
                        if len(u.prefixes) == 1:
                            ttxt = '1 prefix'
                        else:
                            ttxt = str(len(u.prefixes)) + ' prefixes'
                elif isinstance(u,LogUnit):
                    if (round(float(u.conversion.log_base),3) - 2.718) < 0.001:
                        if fmt == 'latex':
                            ttxt += ', log base ' + PrettyPrinter.latex_math('e')
                        if fmt == 'html':
                            ttxt += ', log base <i>e</i>'
                        else:
                            ttxt += ', log base e'
                    else:
                        ttxt += ', log base ' + str(u.conversion.log_base)
                    ttxt += ', multiplier = ' + str(u.conversion.multiplier)
                elif isinstance(u,NonlinearUnit):
                    ttxt += 'non-linear unit'
                    
                if ttxt.startswith(', '):
                    ttxt = ttxt[2:]
                if ttxt != '':
                    txt += ' (' + ttxt + ')'
                
                if u.conversion is not None:
                    try:
                        if isinstance(u,OffsetUnit):
                            g = gummy(0,unit=u)
                        elif isinstance(u,LogUnit):
                            g = gummy(u.conversion.offset,unit=u)
                        else:
                            g = gummy(1,unit=u)
                        gc = g.convert(u.conversion.unit)
                        ctxt = g.tostring(fmt=fmt)
                        if fmt == 'html':
                            ctxt += '&nbsp;=&nbsp;' 
                        else:
                            ctxt += ' = '
                        ctxt += gc.tostring(fmt=fmt)
                        if fmt == 'latex':
                            txt += ', $ ' + ctxt + ' $'
                        else:
                            txt += ', ' + ctxt
                    except:
                        txt += ', ?? = ??'
                elif u is not one:
                    if fmt == 'latex':
                        txt += ', symbol: $ ' + u.tostring(fmt=fmt) + ' $'
                    else:
                        txt += ', symbol: ' + u.tostring(fmt=fmt)
                    
                aliases = u.aliases
                if len(aliases) == 1:
                    txt += ', alias: ' + _mrtxt(aliases.pop(),fmt)
                if len(aliases) > 1:
                    txt += ', aliases: '
                    if u.short_name in aliases:
                        txt += _mrtxt(u.short_name,fmt) + ', '
                        aliases.remove(u.short_name)
                    txt += _mrtxt(', '.join(sorted(aliases,key=str.lower)),fmt)
                    
                saliases = u.shadowed_aliases
                if len(saliases) > 0:
                    if len(aliases) > 0:
                        txt += '; '
                    else:
                        txt += ', '
                    if len(saliases) == 1:
                        txt += 'shadowed alias: ' + _mrtxt(saliases.pop(),fmt)
                    else:
                        txt += 'shadowed aliases: ' + _mrtxt(', '.join(sorted(saliases,key=str.lower)),fmt)
        except:
            txt += '??'
                
        if fmt == 'html' or fmt == 'latex':
            txt += '</li>\n'
        else:
            txt += '\n'
            
    txt = txt[:-1]
    if fmt in ['latex','html']:
        txt += '</ul>'
    
    if not prnt:
        return txt
    
    if fmt == 'latex' and ipython_installed:
            print_markdown(txt)
    elif fmt == 'html' and ipython_installed:
            print_html(txt)
    else:
        print(txt)
        
        
def shadowed_units(fmt=None,prnt=True):
    """
    Lists any units which have a shadowed name or alias.  Units may be shadowed
    if the user has defined a new unit with the same name or alias as an
    existing unit.
    
    Parameters
    ---------
    fmt: {'html','latex','unicode','ascii',`None`},optional
        The output format.  If `None`, then the `gummy.printer` value is used.
        If latex output is selected, Markdown is actually used with the unit
        symbols and conversion displayed using inline LaTeX.

    prnt: `bool`, optional
        If this is `True`, the results are printed.  If it is `False` the results
        are returned as a string.  The default is `True`.
    """
    units = set(Unit._builtin_lib.values()).union(set(Unit._lib.values()))
    units = [u for u in units if len(u.shadowed_aliases) > 0]
    return search_units(fmt=fmt,units=units,prnt=prnt)


def convert(amount,from_unit,to_unit):
    """
    Performs a unit conversion of `amount` in units of `from_unit` to units
    of `to_unit`.
    
    equivalent to ``gummy(amount,from_unit).convert(to_unit)``
    """
    return gummy(amount,unit=from_unit).convert(to_unit)
    
            



# -*- coding: utf-8 -*-
"""
Created on Wed May 10 09:36:41 2017

@author: h parks
"""

from setuptools import setup

__version__ = None
with open('metrolopy/version.py') as f:
    exec(f.read())
    
with open("README.md", "r") as fh:
    long_description = fh.read()
            
setup(name = 'metrolopy',
      version = __version__,
      description = 'tools for dealing with measured quantities: uncertainty propagation and unit conversion',
      long_description=long_description,
      long_description_content_type="text/markdown",
      author = 'Harold Parks, National Research Council Canada',
      author_email = 'parksh@nrc.ca',
      url = 'http://nrc-cnrc.github.io/MetroloPy/',
      packages = ['metrolopy','metrolopy.tests'],
      package_data = {'metrolopy':['license.txt']},
      setup_requires = ['numpy'],
      install_requires=['scipy','matplotlib','pandas'],
      extras_require = {'pretty':['IPython']},
      zip_safe = True,
      classifiers=[
        "Programming Language :: Python :: 3",
        "Operating System :: OS Independent",
        "Development Status :: 4 - Beta",
        "Framework :: Jupyter",
        "Framework :: IPython",
        "License :: OSI Approved :: GNU General Public License v3 (GPLv3)",
        "Intended Audience :: Science/Research",
        "Intended Audience :: Education",
        "Topic :: Scientific/Engineering :: Physics"
        ]   
      )


    
    
